@@ -18,12 +18,15 @@
  *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
+#include <cstdlib>
+#include <algorithm>
+
 #include "charset.h"
 #include "mpdpp.h"
 
 using namespace MPD;
 
-using std::string;
+MPD::Connection Mpd;
 
 const char *MPD::Message::PartOfSongsAdded = "Only part of requested songs' list added to playlist!";
 const char *MPD::Message::FullPlaylist = "Playlist is full!";
@@ -96,10 +99,10 @@ float Connection::Version() const
 	return itsConnection ? itsConnection->version[1] + itsConnection->version[2]*0.1 : 0;
 }
 
-void Connection::SetHostname(const string &host)
+void Connection::SetHostname(const std::string &host)
 {
 	size_t at = host.find("@");
-	if (at != string::npos)
+	if (at != std::string::npos)
 	{
 		itsPassword = host.substr(0, at);
 		itsHost = host.substr(at+1);
@@ -189,7 +192,7 @@ void Connection::UpdateStatus()
 	}
 }
 
-void Connection::UpdateDirectory(const string &path)
+void Connection::UpdateDirectory(const std::string &path)
 {
 	if (isConnected)
 	{
@@ -206,7 +209,7 @@ void Connection::UpdateDirectory(const string &path)
 	}
 }
 
-void Connection::Execute(const string &command) const
+void Connection::Execute(const std::string &command) const
 {
 	if (isConnected)
 	{
@@ -332,7 +335,7 @@ void Connection::ClearPlaylist() const
 	}
 }
 
-void Connection::ClearPlaylist(const string &playlist) const
+void Connection::ClearPlaylist(const std::string &playlist) const
 {
 	if (isConnected)
 	{
@@ -342,13 +345,13 @@ void Connection::ClearPlaylist(const string &playlist) const
 	}
 }
 
-void Connection::AddToPlaylist(const string &path, const Song &s) const
+void Connection::AddToPlaylist(const std::string &path, const Song &s) const
 {
 	if (!s.Empty())
 		AddToPlaylist(path, s.GetFile());
 }
 
-void Connection::AddToPlaylist(const string &path, const string &file) const
+void Connection::AddToPlaylist(const std::string &path, const std::string &file) const
 {
 	if (isConnected)
 	{
@@ -358,7 +361,7 @@ void Connection::AddToPlaylist(const string &path, const string &file) const
 	}
 }
 
-void Connection::Move(const string &path, int from, int to) const
+void Connection::Move(const std::string &path, int from, int to) const
 {
 	if (isConnected)
 	{
@@ -368,7 +371,7 @@ void Connection::Move(const string &path, int from, int to) const
 	}
 }
 
-void Connection::Rename(const string &from, const string &to) const
+void Connection::Rename(const std::string &from, const std::string &to) const
 {
 	if (isConnected)
 	{
@@ -388,8 +391,7 @@ void Connection::GetPlaylistChanges(long long id, SongList &v) const
 			v.reserve(GetPlaylistLength());
 		}
 		mpd_sendPlChangesCommand(itsConnection, id);
-		mpd_InfoEntity *item = NULL;
-		while ((item = mpd_getNextInfoEntity(itsConnection)) != NULL)
+		while (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			if (item->type == MPD_INFO_ENTITY_TYPE_SONG)
 			{
@@ -403,13 +405,12 @@ void Connection::GetPlaylistChanges(long long id, SongList &v) const
 	}
 }
 
-Song Connection::GetSong(const string &path) const
+Song Connection::GetSong(const std::string &path) const
 {
 	if (isConnected)
 	{
 		mpd_sendListallInfoCommand(itsConnection, path.c_str());
-		mpd_InfoEntity *item = NULL;
-		item = mpd_getNextInfoEntity(itsConnection);
+		mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection);
 		Song result = item->info.song;
 		item->info.song = 0;
 		mpd_freeInfoEntity(item);
@@ -430,9 +431,7 @@ Song Connection::GetCurrentSong() const
 	if (isConnected && (GetState() == psPlay || GetState() == psPause))
 	{
 		mpd_sendCurrentSongCommand(itsConnection);
-		mpd_InfoEntity *item = NULL;
-		item = mpd_getNextInfoEntity(itsConnection);
-		if (item)
+		if (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			Song result = item->info.song;
 			item->info.song = 0;
@@ -447,13 +446,12 @@ Song Connection::GetCurrentSong() const
 		return Song();
 }
 
-void Connection::GetPlaylistContent(const string &path, SongList &v) const
+void Connection::GetPlaylistContent(const std::string &path, SongList &v) const
 {
 	if (isConnected)
 	{
 		mpd_sendListPlaylistInfoCommand(itsConnection, path.c_str());
-		mpd_InfoEntity *item = NULL;
-		while ((item = mpd_getNextInfoEntity(itsConnection)) != NULL)
+		while (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			if (item->type == MPD_INFO_ENTITY_TYPE_SONG)
 			{
@@ -509,7 +507,7 @@ void Connection::SetConsume(bool mode) const
 
 void Connection::SetVolume(int vol)
 {
-	if (isConnected)
+	if (isConnected && vol >= 0 && vol <= 100)
 	{
 		mpd_sendSetvolCommand(itsConnection, vol);
 		mpd_finishCommand(itsConnection);
@@ -533,7 +531,7 @@ void Connection::SetCrossfade(int crossfade) const
 	}
 }
 
-int Connection::AddSong(const string &path)
+int Connection::AddSong(const std::string &path)
 {
 	int id = -1;
 	if (isConnected)
@@ -550,14 +548,56 @@ int Connection::AddSong(const string &path)
 				id = 0;
 		}
 		else if (itsErrorHandler)
-				itsErrorHandler(this, MPD_ACK_ERROR_PLAYLIST_MAX, Message::FullPlaylist, NULL);
+				itsErrorHandler(this, MPD_ACK_ERROR_PLAYLIST_MAX, Message::FullPlaylist, itsErrorHandlerUserdata);
 	}
 	return id;
 }
 
 int Connection::AddSong(const Song &s)
 {
-	return !s.Empty() ? (AddSong((!s.IsFromDB() ? "file://" : "") + (s.Localized() ? locale_to_utf_cpy(s.GetFile()) : s.GetFile()))) : -1;
+	return !s.Empty() ? (AddSong((!s.isFromDB() ? "file://" : "") + (s.Localized() ? locale_to_utf_cpy(s.GetFile()) : s.GetFile()))) : -1;
+}
+
+void Connection::Add(const std::string &path) const
+{
+	if (!isConnected)
+		return;
+	mpd_sendAddCommand(itsConnection, path.c_str());
+	mpd_finishCommand(itsConnection);
+}
+
+bool Connection::AddRandomSongs(size_t number)
+{
+	if (!isConnected && !number)
+		return false;
+	
+	TagList files;
+	
+	mpd_sendListallCommand(itsConnection, "/");
+	while (char *file = mpd_getNextTag(itsConnection, MPD_TAG_ITEM_FILENAME))
+	{
+		files.push_back(file);
+		delete [] file;
+	}
+	mpd_finishCommand(itsConnection);
+	
+	if (number > files.size())
+	{
+		if (itsErrorHandler)
+			itsErrorHandler(this, 0, "Requested number of random songs is bigger than size of your library!", itsErrorHandlerUserdata);
+		return false;
+	}
+	else
+	{
+		srand(time(0));
+		std::random_shuffle(files.begin(), files.end());
+		StartCommandsList();
+		TagList::const_iterator it = files.begin()+rand()%(files.size()-number);
+		for (size_t i = 0; i < number && it != files.end(); ++i)
+			AddSong(*it++);
+		CommitCommandsList();
+	}
+	return true;
 }
 
 void Connection::Delete(int pos) const
@@ -580,7 +620,7 @@ void Connection::DeleteID(int id) const
 	}
 }
 
-void Connection::Delete(const string &playlist, int pos) const
+void Connection::Delete(const std::string &playlist, int pos) const
 {
 	if (isConnected)
 	{
@@ -600,7 +640,7 @@ void Connection::StartCommandsList()
 	
 }
 
-void Connection::CommitCommandsList()
+bool Connection::CommitCommandsList()
 {
 	if (isConnected)
 	{
@@ -608,12 +648,13 @@ void Connection::CommitCommandsList()
 		mpd_finishCommand(itsConnection);
 		UpdateStatus();
 		if (GetPlaylistLength() == itsMaxPlaylistLength && itsErrorHandler)
-			itsErrorHandler(this, MPD_ACK_ERROR_PLAYLIST_MAX, Message::FullPlaylist, NULL);
+			itsErrorHandler(this, MPD_ACK_ERROR_PLAYLIST_MAX, Message::FullPlaylist, itsErrorHandlerUserdata);
 		isCommandsListEnabled = 0;
 	}
+	return !CheckForErrors();
 }
 
-void Connection::DeletePlaylist(const string &name) const
+void Connection::DeletePlaylist(const std::string &name) const
 {
 	if (isConnected)
 	{
@@ -623,7 +664,7 @@ void Connection::DeletePlaylist(const string &name) const
 	}
 }
 
-bool Connection::SavePlaylist(const string &name) const
+bool Connection::SavePlaylist(const std::string &name) const
 {
 	if (isConnected)
 	{
@@ -641,7 +682,7 @@ void Connection::GetPlaylists(TagList &v) const
 	{
 		ItemList list;
 		GetDirectory("/", list);
-		for (ItemList::const_iterator it = list.begin(); it != list.end(); it++)
+		for (ItemList::const_iterator it = list.begin(); it != list.end(); ++it)
 		{
 			if (it->type == itPlaylist)
 				v.push_back(it->name);
@@ -654,9 +695,8 @@ void Connection::GetList(TagList &v, mpd_TagItems type) const
 {
 	if (isConnected)
 	{
-		mpd_sendListCommand(itsConnection, type, NULL);
-		char *item;
-		while ((item = mpd_getNextTag(itsConnection, type)) != NULL)
+		mpd_sendListCommand(itsConnection, type, 0);
+		while (char *item = mpd_getNextTag(itsConnection, type))
 		{
 			if (item[0] != 0) // do not push empty item
 				v.push_back(item);
@@ -666,13 +706,13 @@ void Connection::GetList(TagList &v, mpd_TagItems type) const
 	}
 }
 
-void Connection::GetAlbums(const string &artist, TagList &v) const
+void Connection::GetAlbums(const std::string &artist, TagList &v) const
 {
 	if (isConnected)
 	{
-		mpd_sendListCommand(itsConnection, MPD_TABLE_ALBUM, artist.empty() ? NULL : artist.c_str());
-		char *item;
-		while ((item = mpd_getNextAlbum(itsConnection)) != NULL)
+		mpd_sendListCommand(itsConnection, MPD_TABLE_ALBUM, artist.empty() ? 0 : artist.c_str());
+		
+		while (char *item = mpd_getNextAlbum(itsConnection))
 		{
 			if (item[0] != 0) // do not push empty item
 				v.push_back(item);
@@ -697,7 +737,7 @@ void Connection::StartFieldSearch(mpd_TagItems item)
 	}
 }
 
-void Connection::AddSearch(mpd_TagItems item, const string &str) const
+void Connection::AddSearch(mpd_TagItems item, const std::string &str) const
 {
 	// mpd version < 0.14.* doesn't support empty search constraints
 	if (Version() < 14 && str.empty())
@@ -711,8 +751,7 @@ void Connection::CommitSearch(SongList &v) const
 	if (isConnected)
 	{
 		mpd_commitSearch(itsConnection);
-		mpd_InfoEntity *item = NULL;
-		while ((item = mpd_getNextInfoEntity(itsConnection)) != NULL)
+		while (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			if (item->type == MPD_INFO_ENTITY_TYPE_SONG)
 			{
@@ -731,8 +770,7 @@ void Connection::CommitSearch(TagList &v) const
 	if (isConnected)
 	{
 		mpd_commitSearch(itsConnection);
-		char *tag = NULL;
-		while ((tag = mpd_getNextTag(itsConnection, itsSearchedField)) != NULL)
+		while (char *tag = mpd_getNextTag(itsConnection, itsSearchedField))
 		{
 			if (tag[0] != 0) // do not push empty item
 				v.push_back(tag);
@@ -742,13 +780,12 @@ void Connection::CommitSearch(TagList &v) const
 	}
 }
 
-void Connection::GetDirectory(const string &path, ItemList &v) const
+void Connection::GetDirectory(const std::string &path, ItemList &v) const
 {
 	if (isConnected)
 	{
-		mpd_InfoEntity *item = NULL;
 		mpd_sendLsInfoCommand(itsConnection, path.c_str());
-		while ((item = mpd_getNextInfoEntity(itsConnection)) != NULL)
+		while (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			Item i;
 			switch (item->type)
@@ -774,13 +811,12 @@ void Connection::GetDirectory(const string &path, ItemList &v) const
 	}
 }
 
-void Connection::GetDirectoryRecursive(const string &path, SongList &v) const
+void Connection::GetDirectoryRecursive(const std::string &path, SongList &v) const
 {
 	if (isConnected)
 	{
-		mpd_InfoEntity *item = NULL;
 		mpd_sendListallInfoCommand(itsConnection, path.c_str());
-		while ((item = mpd_getNextInfoEntity(itsConnection)) != NULL)
+		while (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			if (item->type == MPD_INFO_ENTITY_TYPE_SONG)
 			{
@@ -794,13 +830,12 @@ void Connection::GetDirectoryRecursive(const string &path, SongList &v) const
 	}
 }
 
-void Connection::GetSongs(const string &path, SongList &v) const
+void Connection::GetSongs(const std::string &path, SongList &v) const
 {
 	if (isConnected)
 	{
-		mpd_InfoEntity *item = NULL;
 		mpd_sendLsInfoCommand(itsConnection, path.c_str());
-		while ((item = mpd_getNextInfoEntity(itsConnection)) != NULL)
+		while (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			if (item->type == MPD_INFO_ENTITY_TYPE_SONG)
 			{
@@ -814,13 +849,12 @@ void Connection::GetSongs(const string &path, SongList &v) const
 	}
 }
 
-void Connection::GetDirectories(const string &path, TagList &v) const
+void Connection::GetDirectories(const std::string &path, TagList &v) const
 {
 	if (isConnected)
 	{
-		mpd_InfoEntity *item = NULL;
 		mpd_sendLsInfoCommand(itsConnection, path.c_str());
-		while ((item = mpd_getNextInfoEntity(itsConnection)) != NULL)
+		while (mpd_InfoEntity *item = mpd_getNextInfoEntity(itsConnection))
 		{
 			if (item->type == MPD_INFO_ENTITY_TYPE_DIRECTORY)
 				v.push_back(item->info.directory->path);
@@ -830,6 +864,37 @@ void Connection::GetDirectories(const string &path, TagList &v) const
 	}
 }
 
+void Connection::GetOutputs(OutputList &v) const
+{
+	if (!isConnected)
+		return;
+	mpd_sendOutputsCommand(itsConnection);
+	while (mpd_OutputEntity *output = mpd_getNextOutput(itsConnection))
+	{
+		v.push_back(std::make_pair(output->name, output->enabled));
+		mpd_freeOutputElement(output);
+	}
+	mpd_finishCommand(itsConnection);
+}
+
+bool Connection::EnableOutput(int id)
+{
+	if (!isConnected)
+		return false;
+	mpd_sendEnableOutputCommand(itsConnection, id);
+	mpd_finishCommand(itsConnection);
+	return !CheckForErrors();
+}
+
+bool Connection::DisableOutput(int id)
+{
+	if (!isConnected)
+		return false;
+	mpd_sendDisableOutputCommand(itsConnection, id);
+	mpd_finishCommand(itsConnection);
+	return !CheckForErrors();
+}
+
 int Connection::CheckForErrors()
 {
 	itsErrorCode = 0;
@@ -862,14 +927,14 @@ int Connection::CheckForErrors()
 
 void MPD::FreeSongList(SongList &l)
 {
-	for (SongList::iterator i = l.begin(); i != l.end(); i++)
+	for (SongList::iterator i = l.begin(); i != l.end(); ++i)
 		delete *i;
 	l.clear();
 }
 
 void MPD::FreeItemList(ItemList &l)
 {
-	for (ItemList::iterator i = l.begin(); i != l.end(); i++)
+	for (ItemList::iterator i = l.begin(); i != l.end(); ++i)
 		delete i->song;
 	l.clear();
 }
