@@ -23,11 +23,13 @@
 extern "C" {
 #endif
 
-char *str_pool_get(const char *value);
+unsigned calc_hash(const char *p);
 
-char *str_pool_dup(const char *value);
+const char *str_pool_get(const char *value);
 
-void str_pool_put(char *value);
+const char *str_pool_dup(const char *value);
+
+void str_pool_put(const char *value);
 
 #ifdef __cplusplus
 }
