@@ -21,18 +21,11 @@
 #include <cstring>
 #include <cstdlib>
 
+#include "error.h"
 #include "window.h"
 
-/// FIXME: dirty workaround.
-#ifdef WIN32
-# define wcwidth(x) 1
-#endif // WIN32
-
 using namespace NCurses;
 
-using std::string;
-using std::wstring;
-
 void NCurses::InitScreen(GNUC_UNUSED const char *window_title, bool enable_colors)
 {
 	const int ColorsTable[] =
@@ -40,7 +33,6 @@ void NCurses::InitScreen(GNUC_UNUSED const char *window_title, bool enable_color
 		COLOR_BLACK, COLOR_RED, COLOR_GREEN, COLOR_YELLOW,
 		COLOR_BLUE, COLOR_MAGENTA, COLOR_CYAN, COLOR_WHITE
 	};
-	setlocale(LC_ALL, "");
 #	ifdef XCURSES
 	Xinitscr(1, const_cast<char **>(&window_title));
 #	else
@@ -56,8 +48,8 @@ void NCurses::InitScreen(GNUC_UNUSED const char *window_title, bool enable_color
 #		else
 		int i = -1;
 #		endif // USE_PDCURSES
-		for (; i < 8; i++)
-			for (int j = 0; j < 8; j++)
+		for (; i < 8; ++i)
+			for (int j = 0; j < 8; ++j)
 				init_pair(num++, ColorsTable[j], i < 0 ? i : ColorsTable[i]);
 	}
 	noecho();
@@ -75,32 +67,36 @@ Window::Window(size_t startx,
 		size_t starty,
 		size_t width,
 		size_t height,
-		const string &title,
+		const std::string &title,
 		Color color,
 		Border border)
 		: itsWindow(0),
 		itsWinBorder(0),
-		itsGetStringHelper(0),
 		itsStartX(startx),
 		itsStartY(starty),
 		itsWidth(width),
 		itsHeight(height),
 		itsWindowTimeout(-1),
+		itsColor(color),
+		itsBgColor(clDefault),
+		itsBaseColor(color),
+		itsBaseBgColor(clDefault),
+		itsBorder(border),
+		itsGetStringHelper(0),
 		itsX(0),
 		itsY(0),
 		itsTitle(title),
-		itsColor(color),
-		itsBaseColor(color),
-		itsBgColor(clDefault),
-		itsBaseBgColor(clDefault),
-		itsBorder(border),
-		itsHistory(0)
+		itsHistory(0),
+		itsBoldCounter(0),
+		itsUnderlineCounter(0),
+		itsReverseCounter(0),
+		itsAltCharsetCounter(0)
 {
 	if (itsStartX > size_t(COLS)
 	||  itsStartY > size_t(LINES)
 	||  itsWidth+itsStartX > size_t(COLS)
 	||  itsHeight+itsStartY > size_t(LINES))
-		throw BadSize();
+		FatalError("Constructed window is bigger than terminal size!");
 	
 	if (itsBorder != brNone)
 	{
@@ -124,27 +120,29 @@ Window::Window(size_t startx,
 	keypad(itsWindow, 1);
 }
 
-Window::Window(const Window &w)
+Window::Window(const Window &w) : itsWindow(dupwin(w.itsWindow)),
+				itsWinBorder(dupwin(w.itsWinBorder)),
+				itsStartX(w.itsStartX),
+				itsStartY(w.itsStartY),
+				itsWidth(w.itsWidth),
+				itsHeight(w.itsHeight),
+				itsWindowTimeout(w.itsWindowTimeout),
+				itsColor(w.itsColor),
+				itsBgColor(w.itsBgColor),
+				itsBaseColor(w.itsBaseColor),
+				itsBaseBgColor(w.itsBaseBgColor),
+				itsBorder(w.itsBorder),
+				itsGetStringHelper(w.itsGetStringHelper),
+				itsX(w.itsX),
+				itsY(w.itsY),
+				itsTitle(w.itsTitle),
+				itsColors(w.itsColors),
+				itsHistory(w.itsHistory),
+				itsBoldCounter(w.itsBoldCounter),
+				itsUnderlineCounter(w.itsUnderlineCounter),
+				itsReverseCounter(w.itsReverseCounter),
+				itsAltCharsetCounter(w.itsAltCharsetCounter)
 {
-	itsWindow = dupwin(w.itsWindow);
-	itsWinBorder = dupwin(w.itsWinBorder);
-	itsGetStringHelper = w.itsGetStringHelper;
-	itsWindowTimeout = w.itsWindowTimeout;
-	itsX = w.itsX;
-	itsY = w.itsY;
-	itsStartX = w.itsStartX;
-	itsStartY = w.itsStartY;
-	itsWidth = w.itsWidth;
-	itsHeight = w.itsHeight;
-	itsTitle = w.itsTitle;
-	itsColors = w.itsColors;
-	itsColor = w.itsColor;
-	itsBaseColor = w.itsBaseColor;
-	itsBgColor = w.itsBgColor;
-	itsBaseBgColor = w.itsBaseBgColor;
-	itsBorder = w.itsBorder;
-	SetColor(itsColor, itsBgColor);
-	keypad(itsWindow, 1);
 }
 
 Window::~Window()
@@ -154,23 +152,23 @@ Window::~Window()
 	delete itsHistory;
 }
 
-void Window::SetColor(Color col, Color background)
+void Window::SetColor(Color fg, Color bg)
 {
-	if (col == clDefault)
-		col = itsBaseColor;
+	if (fg == clDefault)
+		fg = itsBaseColor;
 	
-	if (col != clDefault)
-		wattron(itsWindow, COLOR_PAIR(background*8+col));
+	if (fg != clDefault)
+		wattron(itsWindow, COLOR_PAIR(bg*8+fg));
 	else
 		wattroff(itsWindow, COLOR_PAIR(itsColor));
-	itsColor = col;
-	itsBgColor = background;
+	itsColor = fg;
+	itsBgColor = bg;
 }
 
-void Window::SetBaseColor(Color col, Color background)
+void Window::SetBaseColor(Color fg, Color bg)
 {
-	itsBaseColor = col;
-	itsBaseBgColor = background;
+	itsBaseColor = fg;
+	itsBaseBgColor = bg;
 }
 
 void Window::SetBorder(Border border)
@@ -182,7 +180,7 @@ void Window::SetBorder(Border border)
 		itsStartY--;
 		itsHeight += 2;
 		itsWidth += 2;
-		Recreate();
+		Recreate(itsWidth, itsHeight);
 	}
 	else if (border != brNone && itsBorder == brNone)
 	{
@@ -193,7 +191,7 @@ void Window::SetBorder(Border border)
 		itsStartY++;
 		itsHeight -= 2;
 		itsWidth -= 2;
-		Recreate();
+		Recreate(itsWidth, itsHeight);
 	}
 	else
 	{
@@ -203,25 +201,25 @@ void Window::SetBorder(Border border)
 	itsBorder = border;
 }
 
-void Window::SetTitle(const string &newtitle)
+void Window::SetTitle(const std::string &new_title)
 {
-	if (itsTitle == newtitle)
+	if (itsTitle == new_title)
 	{
 		return;
 	}
-	else if (!newtitle.empty() && itsTitle.empty())
+	else if (!new_title.empty() && itsTitle.empty())
 	{
 		itsStartY += 2;
 		itsHeight -= 2;
-		Recreate();
+		Recreate(itsWidth, itsHeight);
 	}
-	else if (newtitle.empty() && !itsTitle.empty())
+	else if (new_title.empty() && !itsTitle.empty())
 	{
 		itsStartY -= 2;
 		itsHeight += 2;
-		Recreate();
+		Recreate(itsWidth, itsHeight);
 	}
-	itsTitle = newtitle;
+	itsTitle = new_title;
 }
 
 void Window::CreateHistory()
@@ -236,19 +234,19 @@ void Window::DeleteHistory()
 	itsHistory = 0;
 }
 
-void Window::Recreate()
+void Window::Recreate(size_t width, size_t height)
 {
 	delwin(itsWindow);
-	itsWindow = newpad(itsHeight, itsWidth);
+	itsWindow = newpad(height, width);
 	SetTimeout(itsWindowTimeout);
 	SetColor(itsColor, itsBgColor);
 	keypad(itsWindow, 1);
 }
 
-void Window::MoveTo(size_t newx, size_t newy)
+void Window::MoveTo(size_t new_x, size_t new_y)
 {
-	itsStartX = newx;
-	itsStartY = newy;
+	itsStartX = new_x;
+	itsStartY = new_y;
 	if (itsBorder != brNone)
 	{
 		itsStartX++;
@@ -258,7 +256,7 @@ void Window::MoveTo(size_t newx, size_t newy)
 		itsStartY += 2;
 }
 
-void Window::AdjustDimensions(size_t &width, size_t &height)
+void Window::AdjustDimensions(size_t width, size_t height)
 {
 	if (itsBorder != brNone)
 	{
@@ -275,10 +273,10 @@ void Window::AdjustDimensions(size_t &width, size_t &height)
 	itsWidth = width;
 }
 
-void Window::Resize(size_t width, size_t height)
+void Window::Resize(size_t new_width, size_t new_height)
 {
-	AdjustDimensions(width, height);
-	Recreate();
+	AdjustDimensions(new_width, new_height);
+	Recreate(itsWidth, itsHeight);
 }
 
 void Window::ShowBorder() const
@@ -314,32 +312,38 @@ void Window::Refresh()
 	prefresh(itsWindow, 0, 0, itsStartY, itsStartX, itsStartY+itsHeight-1, itsStartX+itsWidth-1);
 }
 
-void Window::Clear(bool)
+void Window::Clear(bool refresh)
 {
 	werase(itsWindow);
-	Window::Refresh();
+	if (refresh)
+		Window::Refresh();
 }
 
-void Window::Hide(char x) const
+void Window::Hide(char ch) const
 {
-	for (size_t i = 0; i < GetHeight(); i++)
-		mvhline(i+GetStartY(), GetStartX(), x, GetWidth());
+	for (size_t i = 0; i < GetHeight(); ++i)
+		mvhline(i+GetStartY(), GetStartX(), ch, GetWidth());
 	refresh();
 }
 
-void Window::Bold(bool bold) const
+void Window::Bold(bool bold_state) const
 {
-	(bold ? wattron : wattroff)(itsWindow, A_BOLD);
+	(bold_state ? wattron : wattroff)(itsWindow, A_BOLD);
 }
 
-void Window::Reverse(bool reverse) const
+void Window::Underline(bool underline_state) const
 {
-	(reverse ? wattron : wattroff)(itsWindow, A_REVERSE);
+	(underline_state ? wattron : wattroff)(itsWindow, A_UNDERLINE);
 }
 
-void Window::AltCharset(bool alt) const
+void Window::Reverse(bool reverse_state) const
 {
-	(alt ? wattron : wattroff)(itsWindow, A_ALTCHARSET);
+	(reverse_state ? wattron : wattroff)(itsWindow, A_REVERSE);
+}
+
+void Window::AltCharset(bool altcharset_state) const
+{
+	(altcharset_state ? wattron : wattroff)(itsWindow, A_ALTCHARSET);
 }
 
 void Window::SetTimeout(int timeout)
@@ -348,9 +352,9 @@ void Window::SetTimeout(int timeout)
 	wtimeout(itsWindow, timeout);
 }
 
-void Window::ReadKey(int &input) const
+void Window::ReadKey(int &read_key) const
 {
-	input = wgetch(itsWindow);
+	read_key = wgetch(itsWindow);
 }
 
 void Window::ReadKey() const
@@ -358,28 +362,7 @@ void Window::ReadKey() const
 	wgetch(itsWindow);
 }
 
-/*void Window::Write(bool cte, const char *format, ...) const
-{
-	va_list list;
-	va_start(list, format);
-	vw_printw(itsWindow, format, list);
-	va_end(list);
-	if (cte)
-		wclrtoeol(itsWindow);
-}
-
-void Window::WriteXY(int x, int y, bool cte, const char *format, ...) const
-{
-	va_list list;
-	va_start(list, format);
-	wmove(itsWindow, y, x);
-	vw_printw(itsWindow, format, list);
-	va_end(list);
-	if (cte)
-		wclrtoeol(itsWindow);
-}*/
-
-string Window::GetString(const string &base, size_t length, size_t width, bool encrypted) const
+std::string Window::GetString(const std::string &base, size_t length, size_t width, bool encrypted) const
 {
 	int input;
 	size_t beginning, maxbeginning, minx, x, real_x, y, maxx, real_maxx;
@@ -397,13 +380,13 @@ string Window::GetString(const string &base, size_t length, size_t width, bool e
 	std::wstring *tmp = &wbase;
 	size_t history_offset = itsHistory && !encrypted ? itsHistory->size() : -1;
 	
-	string tmp_in;
+	std::string tmp_in;
 	wchar_t wc_in;
 	bool gotoend = 1;
 	bool block_scrolling = 0;
 	
 	// disable scrolling if wide chars are used
-	for (wstring::const_iterator it = tmp->begin(); it != tmp->end(); it++)
+	for (std::wstring::const_iterator it = tmp->begin(); it != tmp->end(); ++it)
 		if (wcwidth(*it) > 1)
 			block_scrolling = 1;
 	
@@ -430,7 +413,7 @@ string Window::GetString(const string &base, size_t length, size_t width, bool e
 			if (block_scrolling && maxx >= biggest_x)
 			{
 				size_t i = 0;
-				for (wstring::const_iterator it = tmp->begin(); i < width; it++, real_real_maxx++)
+				for (std::wstring::const_iterator it = tmp->begin(); i < width; ++it, ++real_real_maxx)
 					i += wcwidth(*it);
 			}
 			else
@@ -468,6 +451,8 @@ string Window::GetString(const string &base, size_t length, size_t width, bool e
 		switch (input)
 		{
 			case ERR:
+			case KEY_MOUSE:
+				break;
 			case KEY_UP:
 				if (itsHistory && !encrypted && history_offset > 0)
 				{
@@ -558,7 +543,7 @@ string Window::GetString(const string &base, size_t length, size_t width, bool e
 					break;
 				
 				tmp_in += input;
-				if ((int)mbrtowc(&wc_in, tmp_in.c_str(), MB_CUR_MAX, 0) < 0)
+				if (int(mbrtowc(&wc_in, tmp_in.c_str(), MB_CUR_MAX, 0)) < 0)
 					break;
 				
 				if (wcwidth(wc_in) > 1)
@@ -634,10 +619,22 @@ int Window::Y() const
 	return itsY;
 }
 
-void Window::Scrollable(bool scrollable) const
+bool Window::hasCoords(int &x, int &y)
 {
-	scrollok(itsWindow, scrollable);
-	idlok(itsWindow, scrollable);
+#	ifndef USE_PDCURSES
+	return wmouse_trafo(itsWindow, &y, &x, 0);
+#	else
+	// wmouse_trafo is broken in pdcurses, use our own implementation
+	size_t u_x = x, u_y = y;
+	if (u_x >= itsStartX && u_x < itsStartX+itsWidth
+	&&  u_y >= itsStartY && u_y < itsStartY+itsHeight)
+	{
+		x -= itsStartX;
+		y -= itsStartY;
+		return true;
+	}
+	return false;
+#	endif
 }
 
 size_t Window::GetWidth() const
@@ -676,7 +673,7 @@ size_t Window::GetStartY() const
 	return starty;
 }
 
-const string &Window::GetTitle() const
+const std::string &Window::GetTitle() const
 {
 	return itsTitle;
 }
@@ -720,7 +717,10 @@ void Window::Scroll(Where where)
 Window &Window::operator<<(Colors colors)
 {
 	if (colors.fg == clEnd || colors.bg == clEnd)
+	{
+		*this << clEnd;
 		return *this;
+	}
 	itsColors.push(colors);
 	SetColor(colors.fg, colors.bg);
 	return *this;
@@ -752,38 +752,40 @@ Window &Window::operator<<(Color color)
 
 Window &Window::operator<<(Format format)
 {
-	static int bold_num = 0, reverse_num = 0, altcharset_num = 0;
-	
 	switch (format)
 	{
 		case fmtNone:
-			Bold((bold_num = 0));
-			Reverse((reverse_num = 0));
-			AltCharset((altcharset_num = 0));
+			Bold((itsBoldCounter = 0));
+			Reverse((itsReverseCounter = 0));
+			AltCharset((itsAltCharsetCounter = 0));
 			break;
 		case fmtBold:
-			Bold(++bold_num);
+			Bold(++itsBoldCounter);
 			break;
 		case fmtBoldEnd:
-			// FIXME: No idea why this needs to de disabled at 1 :X
-			// Probably a mistake somewhere in the code, but for now
-			// I was not able to find it
-			if (--bold_num <= 1)
-				Bold((bold_num = 0));
+			if (--itsBoldCounter <= 0)
+				Bold((itsBoldCounter = 0));
+			break;
+		case fmtUnderline:
+			Underline(++itsUnderlineCounter);
+			break;
+		case fmtUnderlineEnd:
+			if (--itsUnderlineCounter <= 0)
+				Underline((itsUnderlineCounter = 0));
 			break;
 		case fmtReverse:
-			Reverse(++reverse_num);
+			Reverse(++itsReverseCounter);
 			break;
 		case fmtReverseEnd:
-			if (--reverse_num <= 0)
-				Reverse((reverse_num = 0));
+			if (--itsReverseCounter <= 0)
+				Reverse((itsReverseCounter = 0));
 			break;
 		case fmtAltCharset:
-			AltCharset(++altcharset_num);
+			AltCharset(++itsAltCharsetCounter);
 			break;
 		case fmtAltCharsetEnd:
-			if (--altcharset_num <= 0)
-				AltCharset((altcharset_num = 0));
+			if (--itsAltCharsetCounter <= 0)
+				AltCharset((itsAltCharsetCounter = 0));
 			break;
 	}
 	return *this;
@@ -837,48 +839,42 @@ Window &Window::operator<<(double d)
 	return *this;
 }
 
-Window &Window::operator<<(const string &s)
+Window &Window::operator<<(const std::string &s)
 {
-	for (string::const_iterator it = s.begin(); it != s.end(); it++)
+	for (std::string::const_iterator it = s.begin(); it != s.end(); ++it)
 		wprintw(itsWindow, "%c", *it);
 	return *this;
 }
 
-Window &Window::operator<<(const wstring &ws)
+Window &Window::operator<<(const std::wstring &ws)
 {
-	for (wstring::const_iterator it = ws.begin(); it != ws.end(); it++)
+	for (std::wstring::const_iterator it = ws.begin(); it != ws.end(); ++it)
 		wprintw(itsWindow, "%lc", *it);
 	return *this;
 }
 
 Window &Window::operator<<(size_t s)
 {
-	wprintw(itsWindow, "%u", s);
+	wprintw(itsWindow, "%zu", s);
 	return *this;
 }
 
-Window * Window::EmptyClone() const
+std::string ToString(const std::wstring &ws)
 {
-	return new Window(GetStartX(), GetStartY(), GetWidth(), GetHeight(), itsTitle, itsBaseColor, itsBorder);
-}
-
-string ToString(const wstring &ws)
-{
-	string result;
-	char *s = new char[MB_CUR_MAX];
-	for (size_t i = 0; i < ws.length(); i++)
+	std::string result;
+	char s[MB_CUR_MAX];
+	for (size_t i = 0; i < ws.length(); ++i)
 	{
 		int n = wcrtomb(s, ws[i], 0);
 		if (n > 0)
 			result.append(s, n);
 	}
-	delete [] s;
 	return result;
 }
 
-wstring ToWString(const string &s)
+std::wstring ToWString(const std::string &s)
 {
-	wstring result;
+	std::wstring result;
 	wchar_t *ws = new wchar_t[s.length()];
 	const char *c_s = s.c_str();
 	int n = mbsrtowcs(ws, &c_s, s.length(), 0);
@@ -888,10 +884,10 @@ wstring ToWString(const string &s)
 	return result;
 }
 
-size_t Window::Length(const wstring &ws)
+size_t Window::Length(const std::wstring &ws)
 {
 	size_t length = 0;
-	for (wstring::const_iterator it = ws.begin(); it != ws.end(); it++)
+	for (std::wstring::const_iterator it = ws.begin(); it != ws.end(); ++it)
 		length += wcwidth(*it);
 	return length;
 }
